# Generated by Django 5.2.5 on 2025-09-12 15:43

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_scrapyd_manager', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='spidergroup',
            name='code',
            field=models.CharField(blank=True, max_length=100, null=True, validators=[django.core.validators.RegexValidator(code='invalid_code', message='代号只能包含英文字母', regex='^[a-zA-Z]*$')], verbose_name='代号'),
        ),
    ]
